"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompaniesService = void 0;
const pagination_service_1 = require("./../pagination/pagination.service");
const pagination_dto_1 = require("./../pagination/pagination.dto");
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
const websocketEvents_1 = require("../websocket/constants/websocketEvents");
const websocket_gateway_1 = require("../websocket/websocket.gateway");
let CompaniesService = class CompaniesService {
    constructor(websocketService, CompanyModel, paginationService) {
        this.websocketService = websocketService;
        this.CompanyModel = CompanyModel;
        this.paginationService = paginationService;
    }
    async fetchAllCompanies(query) {
        let pagination = new pagination_dto_1.PaginationDto();
        let page;
        let totalItemsPage;
        let where = {};
        if (query) {
            if (query["companyCode"]) {
                where["companyCode"] = query["companyCode"];
            }
            if (query["name"]) {
                where["name"] = query["name"];
            }
            if (query["nif"]) {
                where["nif"] = query["nif"];
            }
            if (query["active"] != undefined && (query["active"] == true || query["active"] == 'true')) {
                where["active"] = true;
            }
            if (query["active"] != undefined && (query["active"] == false || query["active"] == 'false')) {
                where["active"] = false;
            }
            if (query["page"]) {
                page = query["page"];
            }
            if (query["totalItemsPage"]) {
                totalItemsPage = query["totalItemsPage"];
            }
        }
        try {
            pagination.totalItems = await this.CompanyModel.countDocuments(where);
            pagination.items = await this.CompanyModel.find(where);
            if (!page && !totalItemsPage) {
                return pagination.items;
            }
            let controlValues = await this.paginationService.controlDefaultPaginationValues(page, totalItemsPage);
            page = controlValues[0];
            totalItemsPage = controlValues[1];
            pagination.totalPages = await this.paginationService.getTotalPagesValue(totalItemsPage, pagination.items);
            page = await this.paginationService.controlIfCurrentPageIsGreaterThanLastPage(page, pagination.totalPages);
            let skip = await this.paginationService.getSkipValue(page, totalItemsPage);
            pagination.items = await this.CompanyModel.find(where).skip(skip).limit(parseInt(totalItemsPage.toString()));
        }
        catch (err) {
            console.error("[fetchAllCompanies] ", err);
            throw new Error(err);
        }
        return pagination;
    }
    async createCompany(newCompany) {
        const company = await this.findCompanyByNif(newCompany.nif);
        if (company) {
            console.log('[createCompany] Company already exist');
            throw new common_1.ConflictException('Company already exist');
        }
        try {
            const company = new this.CompanyModel({
                name: newCompany.name,
                nif: newCompany.nif,
                companyCode: newCompany.companyCode ? newCompany.companyCode : undefined,
                contacts: newCompany.contacts,
                extension: newCompany.extension ? newCompany.extension : {},
                active: newCompany.active == undefined ? false : newCompany.active,
            });
            await company.save();
            console.log('[createCompany] New company created ', company);
            this.notifyWebsockets();
            return company;
        }
        catch (error) {
            console.log('[createCompany] ', error);
            throw new Error(error);
        }
    }
    async updateCompany(nif, newCompany) {
        const company = await this.findCompanyByNif(nif);
        if (!company) {
            console.log('[updateCompany] Company not found');
            throw new common_1.ConflictException('Company not found');
        }
        try {
            const updateCompany = {
                name: newCompany.name,
                nif: nif,
                companyCode: newCompany.companyCode,
                contacts: newCompany.contacts,
                extension: newCompany.extension,
                active: newCompany.active,
            };
            await this.CompanyModel.updateOne({
                nif,
            }, updateCompany);
            console.log('[updateCompany] Company ' + nif + ' updated.');
            this.notifyWebsockets();
            return updateCompany;
        }
        catch (error) {
            console.error('[updateCompany] ', error);
            throw new Error(error);
        }
    }
    async deleteCompany(nif) {
        const company = await this.findCompanyByNif(nif);
        if (!company) {
            console.log('[deleteCompany] Company not found');
            throw new common_1.ConflictException('Company not found');
        }
        let deleted = false;
        try {
            await this.CompanyModel.deleteOne({ nif: nif });
            console.log('[deleteCompany] Company ' + nif + ' deleted.');
            this.notifyWebsockets();
            deleted = true;
        }
        catch (error) {
            console.log('[deleteCompany] ', error);
            throw new Error(error);
        }
        return deleted;
    }
    async findCompany(_id) {
        try {
            return this.CompanyModel.findOne({ _id: _id });
        }
        catch (error) {
            console.error('[findCompany] ', error);
        }
        return null;
    }
    async findCompanyByNif(nif) {
        try {
            return this.CompanyModel.findOne({ nif: nif });
        }
        catch (error) {
            console.error('[findCompanyByNif] ', error);
        }
        return null;
    }
    async notifyWebsockets() {
        try {
            console.log('[WS_COMPANIES] Actualizacion');
            this.websocketService.sendMessageBroadcast(websocketEvents_1.WS_COMPANIES, true);
        }
        catch (err) {
            console.error('[Websocket] Emit error - buffer free');
            console.error(err);
        }
    }
};
CompaniesService = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, common_1.Inject)('MODEL')),
    __metadata("design:paramtypes", [websocket_gateway_1.WebsocketGateway,
        mongoose_1.Model,
        pagination_service_1.PaginationService])
], CompaniesService);
exports.CompaniesService = CompaniesService;
//# sourceMappingURL=companies.service.js.map