"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalConfigurationService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
let GlobalConfigurationService = class GlobalConfigurationService {
    constructor(GlobalConfigurationModel) {
        this.GlobalConfigurationModel = GlobalConfigurationModel;
    }
    async getGlobalConfiguration() {
        try {
            return this.GlobalConfigurationModel.find();
        }
        catch (error) {
            console.error('Error getting global config info -> ', error);
        }
    }
    async getValue(key) {
        try {
            return this.GlobalConfigurationModel.findOne({
                key
            });
        }
        catch (error) {
            console.error('[getValue] Error getting value ', error);
            return null;
        }
    }
    async createGlobalConfig(newConfig) {
        const exists = await this.findConfigurationByKey(newConfig.key);
        if (exists) {
            console.error('[createNewConfig] La configuracion existe');
            throw new Error('La configuracion existe');
        }
        const config = new this.GlobalConfigurationModel({
            key: newConfig.key,
            description: newConfig.description,
            type: newConfig.type,
            value: newConfig.value,
        });
        try {
            await config.save();
            console.error('[createNewConfig] Create new config: ', config);
            return config;
        }
        catch (error) {
            console.error('[createNewConfig] ', error);
            throw new Error(error);
        }
    }
    async updateGlobalConfiguration(newConfiguration) {
        console.log(`[updateGlobalConfiguration] newConfiguration: ${JSON.stringify(newConfiguration)}`);
        if (!newConfiguration || (newConfiguration && newConfiguration.length == 0)) {
            return;
        }
        for (const config of newConfiguration) {
            try {
                let existConfig = await this.getValue(config.key);
                if (!existConfig) {
                    console.log(`[updateGlobalConfiguration] Key ${config.key} not found`);
                    throw new common_1.ConflictException('Key not found');
                }
                existConfig.value = config.value;
                await existConfig.save();
                console.log(`[updateGlobalConfiguration] Key ${config.key} value updated: ${existConfig.value}`);
            }
            catch (error) {
                console.log(`[updateGlobalConfiguration] Error: ${JSON.stringify(error)}`);
                throw new Error(error);
            }
        }
        return this.getGlobalConfiguration();
    }
    async deleteGlobalConfig(key) {
        const existsConfig = await this.findConfigurationByKey(key);
        if (!existsConfig) {
            console.error('[deleteGlobalConfig] La configuracion no existe');
            throw new Error('La configuracion no existe');
        }
        let deleted = false;
        try {
            await existsConfig.delete();
            console.log('[deleteGlobalConfig] Config ' + key + ' deleted');
            deleted = true;
        }
        catch (err) {
            console.error("[deleteGlobalConfig] ", err);
            throw new Error(err);
        }
        return deleted;
    }
    findConfigurationByKey(key) {
        return this.GlobalConfigurationModel.findOne({ key });
    }
};
GlobalConfigurationService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)('MODEL')),
    __metadata("design:paramtypes", [mongoose_1.Model])
], GlobalConfigurationService);
exports.GlobalConfigurationService = GlobalConfigurationService;
//# sourceMappingURL=global-configuration.service.js.map