"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.userSchema = void 0;
const bcrypt = require("bcrypt");
const mongoose_1 = require("mongoose");
exports.userSchema = new mongoose_1.Schema({
    userName: {
        type: String,
        required: true,
        unique: true,
        lowercase: true,
    },
    password: {
        type: String,
        required: true,
    },
    email: {
        type: String,
        unique: true,
        required: false,
        lowercase: true,
        trim: true,
    },
    role: {
        type: mongoose_1.Types.ObjectId,
        ref: 'Role',
        required: true,
    },
    active: {
        type: Boolean,
        required: false,
        default: true,
    },
    pwdRecoveryToken: {
        type: String,
        required: false,
        default: null,
    },
    pwdRecoveryDate: {
        type: Date,
        required: false,
        default: null
    },
    company: {
        type: mongoose_1.Types.ObjectId,
        ref: 'Company',
        required: false,
        default: null,
    },
    extension: {
        type: mongoose_1.Schema.Types.Mixed,
        require: false
    },
    typeObj: {
        type: String,
        required: false,
        default: 'User'
    }
}, {
    timestamps: true,
});
exports.userSchema.pre('save', async function () {
    const salt = await bcrypt.genSalt(10);
    const hash = await bcrypt.hash(this.password, salt);
    this.password = hash;
});
//# sourceMappingURL=user.model.js.map