"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PermissionsService = void 0;
const pagination_service_1 = require("./../pagination/pagination.service");
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
const pagination_dto_1 = require("../pagination/pagination.dto");
const websocketEvents_1 = require("../websocket/constants/websocketEvents");
const websocket_gateway_1 = require("../websocket/websocket.gateway");
let PermissionsService = class PermissionsService {
    constructor(paginationService, websocketService, PermissionModel) {
        this.paginationService = paginationService;
        this.websocketService = websocketService;
        this.PermissionModel = PermissionModel;
    }
    async fetchAllPermissions(query) {
        let pagination = new pagination_dto_1.PaginationDto();
        let page;
        let totalItemsPage;
        let where = {};
        if (query["name"]) {
            where["name"] = query["name"];
        }
        if (query["page"]) {
            page = query["page"];
        }
        if (query["totalItemsPage"]) {
            totalItemsPage = query["totalItemsPage"];
        }
        try {
            pagination.totalItems = await this.PermissionModel.countDocuments(where);
            pagination.items = await this.PermissionModel.find(where);
            if (!page && !totalItemsPage) {
                return pagination.items;
            }
            let controlValues = await this.paginationService.controlDefaultPaginationValues(page, totalItemsPage);
            page = controlValues[0];
            totalItemsPage = controlValues[1];
            pagination.totalPages = await this.paginationService.getTotalPagesValue(totalItemsPage, pagination.items);
            page = await this.paginationService.controlIfCurrentPageIsGreaterThanLastPage(page, pagination.totalPages);
            let skip = await this.paginationService.getSkipValue(page, totalItemsPage);
            pagination.items = await this.PermissionModel.find(where).skip(skip).limit(parseInt(totalItemsPage.toString()));
        }
        catch (err) {
            console.error("[fetchAllPermissions] ", err);
            throw new Error(err);
        }
        return pagination;
    }
    async createPermission(permission) {
        const permissionExists = await this.fetchPermission(permission.name);
        if (permissionExists) {
            console.log("[createPermission] Permission " + permission.name + " already exist");
            throw new common_1.ConflictException('Permission ' + permission.name + ' already exist');
        }
        try {
            const newPermission = new this.PermissionModel({
                name: permission.name,
            });
            await newPermission.save();
            console.log("[createPermission] Create new permission: ", newPermission);
            this.notifyWebsockets();
            return newPermission;
        }
        catch (err) {
            console.log("[createPermission] ", err);
            throw new Error(err);
        }
    }
    async updatePermission(name, permission) {
        const existingPermission = await this.fetchPermission(name);
        if (!existingPermission) {
            console.error("[updatePermission] Permission " + name + " not found");
            throw new common_1.ConflictException('Permission ' + name + ' not found');
        }
        const existingPermissionUpdate = await this.fetchPermission(permission.name);
        if (existingPermissionUpdate && (existingPermissionUpdate.name != existingPermission.name)) {
            console.log("[updatePermission] Permission " + permission.name + " already exist");
            throw new common_1.ConflictException('Permission ' + permission.name + ' already exist');
        }
        try {
            Object.assign(existingPermission, permission);
            await existingPermission.save();
            console.log('[updatePermission] Permission ' + name + ' updated to ' + permission.name);
            this.notifyWebsockets();
            return existingPermission;
        }
        catch (err) {
            console.log("[updatePermission] ", err);
            throw new Error(err);
        }
        return null;
    }
    async deletePermission(name) {
        const permission = await this.fetchPermission(name);
        if (!permission) {
            console.log("[deletePermission] Permission " + name + " not found");
            throw new common_1.ConflictException('Permission ' + name + ' not found');
        }
        let deleted = false;
        try {
            await permission.delete();
            console.log('[deletePermission] Role ' + permission + ' deleted');
            this.notifyWebsockets();
            deleted = true;
        }
        catch (err) {
            console.error("[deletePermission] ", err);
            throw new Error(err);
        }
        return deleted;
    }
    async fetchPermission(name) {
        try {
            const permission = await this.PermissionModel.findOne({ name });
            return permission;
        }
        catch (err) {
            console.error("[fetchPermission] ", err);
            return null;
        }
    }
    async notifyWebsockets() {
        try {
            console.log("[WS_PERMISSIONS] Actualizacion");
            this.websocketService.sendMessageBroadcast(websocketEvents_1.WS_PERMISSIONS, true);
        }
        catch (err) {
            console.error('[Websocket] Emit error - buffer free');
            console.error(err);
            return null;
        }
    }
};
PermissionsService = __decorate([
    (0, common_1.Injectable)(),
    __param(2, (0, common_1.Inject)('MODEL')),
    __metadata("design:paramtypes", [pagination_service_1.PaginationService,
        websocket_gateway_1.WebsocketGateway,
        mongoose_1.Model])
], PermissionsService);
exports.PermissionsService = PermissionsService;
//# sourceMappingURL=permissions.service.js.map