"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RolesService = void 0;
const pagination_service_1 = require("./../pagination/pagination.service");
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
const permissions_service_1 = require("../permissions/permissions.service");
const users_service_1 = require("../users/users.service");
const websocketEvents_1 = require("../websocket/constants/websocketEvents");
const websocket_gateway_1 = require("../websocket/websocket.gateway");
const pagination_dto_1 = require("../pagination/pagination.dto");
let RolesService = class RolesService {
    constructor(paginationService, permissionService, usersService, websocketService, RoleModel) {
        this.paginationService = paginationService;
        this.permissionService = permissionService;
        this.usersService = usersService;
        this.websocketService = websocketService;
        this.RoleModel = RoleModel;
    }
    async fetchAllRoles(query) {
        let pagination = new pagination_dto_1.PaginationDto();
        let page;
        let totalItemsPage;
        let where = {};
        where.name = { $ne: 'SUPERADMIN' };
        if (query["name"]) {
            where.name = { $ne: 'SUPERADMIN', $eq: query["name"] };
        }
        if (query["page"]) {
            page = query["page"];
        }
        if (query["totalItemsPage"]) {
            totalItemsPage = query["totalItemsPage"];
        }
        try {
            pagination.totalItems = await this.RoleModel.countDocuments(where);
            pagination.items = await this.RoleModel.find(where).populate({
                path: 'permissions',
                model: 'Permission',
            });
            ;
            if (!page && !totalItemsPage) {
                return pagination.items;
            }
            let controlValues = await this.paginationService.controlDefaultPaginationValues(page, totalItemsPage);
            page = controlValues[0];
            totalItemsPage = controlValues[1];
            pagination.totalPages = await this.paginationService.getTotalPagesValue(totalItemsPage, pagination.items);
            page = await this.paginationService.controlIfCurrentPageIsGreaterThanLastPage(page, pagination.totalPages);
            let skip = await this.paginationService.getSkipValue(page, totalItemsPage);
            pagination.items = await this.RoleModel.find(where).skip(skip).limit(parseInt(totalItemsPage.toString())).populate({
                path: 'permissions',
                model: 'Permission',
            });
        }
        catch (err) {
            console.error("[fetchAllRoles] ", err);
            throw new Error(err);
        }
        return pagination;
    }
    async createRole(role) {
        const roleExists = await this.findRole(role.name);
        if (roleExists) {
            console.log("[createRole] Role " + role.name + " already exist");
            throw new common_1.ConflictException("Role " + role.name + " already exist");
        }
        const newRole = new this.RoleModel({
            name: role.name,
            permissions: []
        });
        for (const p of role.permissions) {
            const permission = await this.permissionService.fetchPermission(p.name);
            if (!permission) {
                console.log("[createRole] " + permission.name + " not found");
                throw new common_1.ConflictException("Permission " + permission.name + " not found");
            }
            else {
                newRole.permissions.push(permission._id);
            }
        }
        try {
            await newRole.save();
            console.log("[createRole] New role created: ", newRole);
            this.notifyWebsockets();
            return newRole;
        }
        catch (err) {
            console.error("[createRole] ", err);
            throw new Error(err);
        }
    }
    async updateRole(oldRoleName, newRole) {
        const existsRole = await this.findRole(oldRoleName);
        if (!existsRole) {
            console.log("[updateRole] Role " + oldRoleName + " not found");
            throw new common_1.ConflictException('Role ' + oldRoleName + ' not found');
        }
        if (oldRoleName != newRole.name) {
            const existingRoleUpdate = await this.fetchRole(newRole.name);
            if (existingRoleUpdate) {
                console.log("[updateRole] Role " + newRole.name + " already exist");
                throw new common_1.ConflictException('Role ' + newRole.name + ' already exist');
            }
        }
        const permissions = [];
        for (const p of newRole.permissions) {
            const permission = await this.permissionService.fetchPermission(p.name);
            if (!permission) {
                console.log("[updateRole] Permission " + permission.name + " not found");
                throw new common_1.ConflictException("Permission " + permission.name + " not found");
            }
            permissions.push(permission);
        }
        try {
            await this.RoleModel.updateOne({
                name: oldRoleName,
            }, {
                name: newRole.name,
                permissions
            });
            const roleUpdated = await this.findRole(newRole.name);
            console.log("[updateRole] Role updated: ", roleUpdated);
            this.notifyWebsockets();
            return roleUpdated;
        }
        catch (err) {
            console.error("[updateRole] ", err);
            throw new Error(err);
        }
    }
    async deleteRole(roleName) {
        const role = await this.RoleModel.findOne({ name: roleName });
        if (!role) {
            console.log('[deleteRole] Role ' + roleName + ' not found');
            throw new common_1.ConflictException('Role ' + roleName + ' not found');
        }
        const hasUsers = await this.usersService.roleHasUsers(role.name);
        if (hasUsers) {
            console.log('[deleteRole] Role ' + roleName + ' contains users');
            throw new common_1.ConflictException('Role ' + roleName + ' contains users');
        }
        let deleted = false;
        try {
            await role.delete();
            console.log('[deleteRole] Role ' + roleName + ' deleted');
            this.notifyWebsockets();
            deleted = true;
        }
        catch (err) {
            console.error("[deleteRole] ", err);
            throw new Error(err);
        }
        return deleted;
    }
    async findRole(role) {
        try {
            return this.RoleModel.findOne({ name: role });
        }
        catch (err) {
            console.error("[findRole] ", err);
        }
        return null;
    }
    async fetchRole(role) {
        try {
            return this.RoleModel.findOne({ name: role }).populate('permissions').populate({
                path: 'permissions',
                model: 'Permission',
            });
        }
        catch (err) {
            console.error("[fetchRole] ", err);
        }
        return null;
    }
    async notifyWebsockets() {
        try {
            console.log("[WS_ROLES] Actualizacion");
            this.websocketService.sendMessageBroadcast(websocketEvents_1.WS_ROLES, true);
        }
        catch (err) {
            console.error('[Websocket] Emit error - buffer free');
            console.error(err);
        }
    }
};
RolesService = __decorate([
    (0, common_1.Injectable)(),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => users_service_1.UsersService))),
    __param(4, (0, common_1.Inject)('MODEL')),
    __metadata("design:paramtypes", [pagination_service_1.PaginationService,
        permissions_service_1.PermissionsService,
        users_service_1.UsersService,
        websocket_gateway_1.WebsocketGateway,
        mongoose_1.Model])
], RolesService);
exports.RolesService = RolesService;
//# sourceMappingURL=roles.service.js.map