"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SftpService = void 0;
const common_1 = require("@nestjs/common");
let SftpService = class SftpService {
    constructor(options) {
        this.options = options;
        this.Client = require('ssh2-sftp-client');
        this.configurationSftp = options;
    }
    async getSftpConfig() {
        return this.configurationSftp;
    }
    async setSftpConfig(options) {
        this.configurationSftp = options;
    }
    async putFile(sftpRequest) {
        let puttedFile = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.fastPut(sftpRequest.path, sftpRequest.newPath);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[putFile] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                puttedFile = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[putFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return puttedFile;
    }
    async getFile(sftpRequest) {
        let gettedFile = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.exists(sftpRequest.path);
        })
            .then((data) => {
            if (data == false) {
                if (sftpRequest.verbose) {
                    console.log(`[getFile] Remote file does not exists`);
                }
                throw new common_1.ConflictException('Remote file does not exists.');
            }
        })
            .then(async () => {
            return await this.sftp.fastGet(sftpRequest.path, sftpRequest.newPath);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[getFile] ${JSON.stringify(data)}`);
            }
            if (data) {
                gettedFile = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[getFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return gettedFile;
    }
    async moveFile(sftpRequest) {
        let movedFile = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.exists(sftpRequest.path);
        })
            .then((data) => {
            if (data == false) {
                if (sftpRequest.verbose) {
                    console.log(`[moveFile] Remote file does not exists.`);
                }
                throw new common_1.ConflictException('Remote file does not exists.');
            }
        })
            .then(async () => {
            return await this.sftp.rcopy(sftpRequest.path, sftpRequest.newPath);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[moveFile] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                movedFile = true;
            }
        })
            .then(async () => {
            return await this.sftp.delete(sftpRequest.path);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[moveFile] ${JSON.stringify(data)}`);
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[moveFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return movedFile;
    }
    async deleteFile(sftpRequest) {
        let deletedFile = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.exists(sftpRequest.path);
        })
            .then((data) => {
            if (data == false) {
                if (sftpRequest.verbose) {
                    console.log(`[deleteFile] Remote file does not exists.`);
                }
                throw new common_1.ConflictException('Remote file does not exists.');
            }
        })
            .then(async () => {
            return await this.sftp.delete(sftpRequest.path);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[deleteFile] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                deletedFile = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[deleteFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return deletedFile;
    }
    async renameFile(sftpRequest) {
        let renamedFile = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.exists(sftpRequest.path);
        })
            .then((data) => {
            if (data == false) {
                if (sftpRequest.verbose) {
                    console.log(`[renameFile] Remote file does not exists.`);
                }
                throw new common_1.ConflictException('Remote file does not exists.');
            }
        })
            .then(async () => {
            return await this.sftp.rename(sftpRequest.path, sftpRequest.newPath);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[renameFile] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                renamedFile = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[renameFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return renamedFile;
    }
    async existsFile(sftpRequest) {
        let exists = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.exists(sftpRequest.path);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[existsFile] ${JSON.stringify(data)}`);
            }
            if (data != false)
                exists = true;
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[existsFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return exists;
    }
    async list(sftpRequest) {
        const list = [];
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.list(sftpRequest.path);
        })
            .then(async (data) => {
            if (sftpRequest.verbose) {
                console.log(`[list] ${JSON.stringify(data)}`);
            }
            if (data) {
                data.forEach(async (element) => {
                    list.push({
                        name: element.name,
                        type: element.type == '-' ? 'file' : 'directory',
                    });
                });
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .then(async () => {
            return list;
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[list] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return list;
    }
    async getBase64FromRemoteFile(sftpRequest) {
        let base64 = '';
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.get(sftpRequest.path);
        })
            .then(async (data) => {
            if (sftpRequest.verbose) {
                console.log(`[getBase64FromRemoteFile] ${JSON.stringify(data)}`);
            }
            if (data) {
                base64 = Buffer.from(data).toString('base64');
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .then(async () => {
            return base64;
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[getBase64FromRemoteFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return base64;
    }
    async getBufferFromRemoteFile(sftpRequest) {
        let buffer = null;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.get(sftpRequest.path);
        })
            .then(async (data) => {
            if (sftpRequest.verbose) {
                console.log(`[getBufferFromRemoteFile] ${JSON.stringify(data)}`);
            }
            if (data) {
                buffer = JSON.parse(JSON.stringify(data)).data;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .then(async () => {
            return buffer;
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[getBufferFromRemoteFile] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return buffer;
    }
    async checkIfFileIsDirectory(sftpRequest) {
        let isDirectory = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.stat(sftpRequest.path);
        })
            .then(async (data) => {
            if (sftpRequest.verbose) {
                console.log(`[checkIfFileIsDirectory] ${JSON.stringify(data)}`);
            }
            if (data) {
                isDirectory = JSON.parse(JSON.stringify(data)).isDirectory;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .then(async () => {
            return isDirectory;
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[checkIfFileIsDirectory] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return isDirectory;
    }
    async uploadDir(sftpRequest) {
        let uploadedDir = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.uploadDir(sftpRequest.path, sftpRequest.newPath);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[uploadDir] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                uploadedDir = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[uploadDir] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return uploadedDir;
    }
    async downloadDir(sftpRequest) {
        let downloadedDir = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.downloadDir(sftpRequest.path, sftpRequest.newPath);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[downloadDir] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                downloadedDir = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[downloadDir] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return downloadedDir;
    }
    async createFolder(sftpRequest) {
        let createdFolder = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.mkdir(sftpRequest.path, sftpRequest.recursive);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[createFolder] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                createdFolder = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[createFolder] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return createdFolder;
    }
    async deleteFolder(sftpRequest) {
        let deletedFolder = false;
        this.sftp = new this.Client();
        await this.sftp
            .connect(this.configurationSftp)
            .then(async () => {
            return await this.sftp.rmdir(sftpRequest.path, sftpRequest.recursive);
        })
            .then((data) => {
            if (sftpRequest.verbose) {
                console.log(`[deleteFolder] ${JSON.stringify(data)}`);
            }
            if (data != false) {
                deletedFolder = true;
            }
        })
            .then(async () => {
            return await this.closeConnection();
        })
            .catch(async (err) => {
            if (sftpRequest.verbose) {
                console.error(`[deleteFolder] ${err.message}`);
            }
            return await this.closeConnection();
        });
        return deletedFolder;
    }
    async closeConnection() {
        try {
            await this.sftp.client.removeAllListeners('error');
            await this.sftp.end();
            return;
        }
        catch (err) {
            console.error(`[closeConnection] ${err.message}`);
        }
    }
};
SftpService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)('CONFIG_OPTIONS')),
    __metadata("design:paramtypes", [Object])
], SftpService);
exports.SftpService = SftpService;
//# sourceMappingURL=sftp.service.js.map