"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UsersService = void 0;
const pagination_dto_1 = require("./../pagination/pagination.dto");
const pagination_service_1 = require("./../pagination/pagination.service");
const common_1 = require("@nestjs/common");
const bcrypt = require("bcrypt");
const mongoose_1 = require("mongoose");
const roles_service_1 = require("../roles/roles.service");
const websocketEvents_1 = require("../websocket/constants/websocketEvents");
const websocket_gateway_1 = require("../websocket/websocket.gateway");
const companies_service_1 = require("../companies/companies.service");
let UsersService = class UsersService {
    constructor(paginationService, roleService, companiesService, websocketService, UserModel) {
        this.paginationService = paginationService;
        this.roleService = roleService;
        this.companiesService = companiesService;
        this.websocketService = websocketService;
        this.UserModel = UserModel;
    }
    async fetchAllUsers(query) {
        let pagination = new pagination_dto_1.PaginationDto();
        let page;
        let totalItemsPage;
        let where = {};
        where.userName = { $ne: 'SUPERADMIN' };
        if (query["userName"]) {
            where.userName = { $ne: 'SUPERADMIN', $eq: query["userName"] };
        }
        if (query["email"]) {
            where.email = { $eq: query["email"] };
        }
        if (query["role"]) {
            const filterRole = await this.roleService.findRole(query["role"]);
            if (filterRole) {
                where.role = { $eq: filterRole };
            }
        }
        if (query["active"] == "true" || query["active"] == "false") {
            where.active = { $eq: query["active"] };
        }
        if (query["company"]) {
            const filterCompany = await this.companiesService.findCompany(query["company"]);
            if (filterCompany) {
                where.company = { $eq: filterCompany };
            }
        }
        if (query["page"]) {
            page = query["page"];
        }
        if (query["totalItemsPage"]) {
            totalItemsPage = query["totalItemsPage"];
        }
        try {
            pagination.totalItems = await this.UserModel.countDocuments(where);
            pagination.items = await this.UserModel.find(where)
                .populate('company')
                .populate({
                path: 'role',
                populate: {
                    path: 'permissions',
                    model: 'Permission',
                },
            });
            ;
            if (!page && !totalItemsPage) {
                return pagination.items;
            }
            let controlValues = await this.paginationService.controlDefaultPaginationValues(page, totalItemsPage);
            page = controlValues[0];
            totalItemsPage = controlValues[1];
            pagination.totalPages = await this.paginationService.getTotalPagesValue(totalItemsPage, pagination.items);
            page = await this.paginationService.controlIfCurrentPageIsGreaterThanLastPage(page, pagination.totalPages);
            let skip = await this.paginationService.getSkipValue(page, totalItemsPage);
            pagination.items = await this.UserModel.find(where).skip(skip).limit(parseInt(totalItemsPage.toString()))
                .populate('company')
                .populate({
                path: 'role',
                populate: {
                    path: 'permissions',
                    model: 'Permission',
                },
            });
        }
        catch (err) {
            console.error("[fetchAllUsers] ", err);
            throw new Error(err);
        }
        return pagination;
    }
    async createUser(newUser) {
        console.log(JSON.stringify(newUser));
        const user = await this.findUser(newUser.userName);
        if (user) {
            console.log("[createUser] User " + newUser.userName + " already exist");
            throw new common_1.ConflictException("User " + newUser.userName + ' already exist');
        }
        if (newUser.email && newUser.email.length > 0) {
            const emailExist = await this.UserModel.findOne({ email: newUser.email });
            if (emailExist) {
                console.log("[createUser] Email " + newUser.email + " already registered");
                throw new common_1.ConflictException("Email " + newUser.email + ' already registered');
            }
        }
        const roleExists = await this.roleService.fetchRole(newUser.role.name);
        if (!roleExists) {
            console.log('[createUser] Role ' + newUser.role.name + ' not exist');
            throw new common_1.ConflictException('Role ' + newUser.role.name + ' not exist');
        }
        let company = null;
        if (newUser.company && newUser.company.companyCode) {
            company = await this.companiesService.findCompany(newUser.company.companyCode);
            if (!company) {
                console.log('[createUser] Company does not exist.');
                throw new common_1.ConflictException('Company does not exist.');
            }
        }
        try {
            const user = new this.UserModel({
                userName: newUser.userName,
                password: newUser.password,
                email: newUser.email,
                role: roleExists['_id'],
                active: newUser.active,
                company: company,
                extension: newUser.extension ? newUser.extension : {},
            });
            await user.save();
            console.log('[createUser] New user created ', user);
            this.notifyWebsockets();
            user.password = undefined;
            return user;
        }
        catch (err) {
            console.error("[createUser] ", err);
            throw new Error(err);
        }
    }
    async updateUser(userName, newUser) {
        const user = await this.findUser(userName);
        if (!user) {
            console.log('[updateUser] User ' + userName + ' not found');
            throw new common_1.ConflictException('User ' + userName + ' not found');
        }
        if (newUser.password) {
            let res = await bcrypt.compare(newUser.password, user.password);
            if (!res) {
                console.log('[updateUser] User ' + userName + ' incorrect password');
                throw new common_1.ConflictException('User ' + userName + ' incorrect password');
            }
        }
        const roleExists = await this.roleService.fetchRole(newUser.role.name);
        if (!roleExists) {
            console.log('[updateUser] Role ' + newUser.role.name + ' not exist');
            throw new common_1.ConflictException('Role ' + newUser.role.name + ' not exist');
        }
        if (newUser.email && user.email && newUser.email != user.email) {
            const emailExist = await this.UserModel.findOne({ email: newUser.email });
            if (emailExist) {
                console.log("[createUser] Email " + newUser.email + " already registered");
                throw new common_1.ConflictException("Email " + newUser.email + ' already registered');
            }
        }
        let company = null;
        if (newUser.company && newUser.company.nif) {
            company = await this.companiesService.findCompanyByNif(newUser.company.nif);
            if (!company) {
                console.log('[updateUser] Company does not exist.');
                throw new common_1.ConflictException('Company does not exist.');
            }
        }
        try {
            let updateUser = {
                userName: newUser.userName,
                email: newUser.email,
                role: roleExists['_id'],
                active: newUser.active,
                company: company != null ? company : user.company,
                extension: newUser.extension ? newUser.extension : {},
            };
            if (newUser.newPassword) {
                newUser.newPassword = await this.encryptPassword(newUser.newPassword);
                updateUser.password = newUser.newPassword;
            }
            await this.UserModel.updateOne({
                userName,
            }, updateUser);
            console.log('[updateUser] User ' + userName + ' updated.');
            this.notifyWebsockets();
            return user;
        }
        catch (err) {
            console.error("[updateUser] ", err);
            throw new Error(err);
        }
    }
    async deleteUser(userName) {
        const user = await this.findUser(userName);
        if (!user) {
            console.log('[deleteUser] User ' + userName + ' not found');
            throw new common_1.ConflictException('User ' + userName + ' not found');
        }
        try {
            await this.UserModel.deleteOne({
                userName,
            });
            console.log('[deleteUser] User ' + userName + ' deleted.');
            this.notifyWebsockets();
            return user;
        }
        catch (err) {
            console.error("[deleteUser] ", err);
            throw new Error(err);
        }
    }
    async activateUser(userName) {
        const user = await this.findUser(userName);
        if (!user) {
            console.log('[activateUser] User ' + userName + ' not found');
            throw new common_1.ConflictException('User ' + userName + ' not found');
        }
        try {
            let nextState = false;
            if (!user.active) {
                nextState = true;
            }
            user.active = nextState;
            await this.UserModel.updateOne({
                userName,
            }, {
                userName: user.userName,
                password: user.password,
                email: user.email,
                role: user.role,
                active: user.active,
            });
            console.log('[activateUser] User ' + userName + (user.active ? ' activado' : ' desactivado'));
            this.notifyWebsockets();
            return user;
        }
        catch (err) {
            console.error("[activateUser] ", err);
            throw new Error(err);
        }
    }
    async pwdRecovery(email) {
        if (!email) {
            console.log('[pwdRecovery] Email ' + email + ' not informed');
            throw new common_1.ConflictException('Email ' + email + ' not informed');
        }
        let user = await this.findUserByEmail(email);
        if (!user) {
            console.log('[pwdRecovery] User with email ' + email + ' not exist');
            throw new common_1.ConflictException('User with email ' + email + ' not exist');
        }
        try {
            user.pwdRecoveryDate = new Date();
            user.pwdRecoveryToken = await this.generatePasswordRecoveryToken();
            const userName = user.userName;
            await this.UserModel.updateOne({
                userName,
            }, user);
            console.log('[pwdRecovery] User ' + userName + ' updated.');
            this.notifyWebsockets();
        }
        catch (err) {
            console.error("[pwdRecovery] ", err);
            throw new Error(err);
        }
        finally {
            return user;
        }
    }
    async updateUserPassword(userName, newUser) {
        const user = await this.findUser(userName);
        if (!user) {
            console.log('[updateUserPassword] User ' + userName + ' not found');
            throw new common_1.ConflictException('User ' + userName + ' not found');
        }
        try {
            newUser.newPassword = await this.encryptPassword(newUser.newPassword);
            let updateUser = user;
            updateUser.password = newUser.newPassword;
            updateUser.pwdRecoveryToken = null;
            updateUser.pwdRecoveryDate = null;
            await this.UserModel.updateOne({
                userName,
            }, updateUser);
            console.log('[updateUserPassword] User ' + userName + ' updated.');
            this.notifyWebsockets();
            return user;
        }
        catch (err) {
            console.error("[updateUserPassword] ", err);
            throw new Error(err);
        }
    }
    async fetchUserByPwdRecovery(pwdRecoveryToken) {
        try {
            return this.UserModel
                .findOne({ pwdRecoveryToken: pwdRecoveryToken }, { "password": 0 })
                .populate('company')
                .populate({
                path: 'role',
                populate: {
                    path: 'permissions',
                    model: 'Permission',
                },
            });
        }
        catch (err) {
            console.error("[fetchUserByPwdRecovery] ", err);
            throw new Error(err);
        }
    }
    async findUser(userName) {
        try {
            return this.UserModel.findOne({ userName: userName.toLowerCase() });
        }
        catch (err) {
            console.error("[findUser] ", err);
            throw new Error(err);
        }
    }
    async fetchUser(userName) {
        try {
            return this.UserModel.findOne({ userName: userName.toLocaleLowerCase() })
                .populate('company')
                .populate({
                path: 'role',
                populate: {
                    path: 'permissions',
                    model: 'Permission',
                },
            });
        }
        catch (err) {
            console.error("[fetchUser] ", err);
            throw new Error(err);
        }
    }
    async findUserByEmail(email) {
        try {
            return this.UserModel.findOne({ email: email.toLowerCase() });
        }
        catch (err) {
            console.error("[findUserByEmail] ", err);
            throw new Error(err);
        }
    }
    async roleHasUsers(name) {
        try {
            const users = await this.UserModel.aggregate([
                {
                    $lookup: {
                        from: 'roles',
                        localField: 'role',
                        foreignField: '_id',
                        as: 'role',
                    },
                },
                {
                    $match: {
                        'role.name': name,
                    },
                },
            ]);
            return users.length > 0;
        }
        catch (err) {
            console.error("[roleHasUsers] ", err);
            throw new Error(err);
        }
    }
    async encryptPassword(password) {
        try {
            const salt = await bcrypt.genSalt(10);
            const hash = await bcrypt.hash(password, salt);
            return hash;
        }
        catch (err) {
            console.error("[encryptPassword] ", err);
            throw new Error(err);
        }
    }
    async generatePasswordRecoveryToken() {
        try {
            let text = "";
            const possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
            for (var i = 0; i < 64; i++) {
                text += possible.charAt(Math.floor(Math.random() * possible.length));
            }
            return text;
        }
        catch (err) {
            console.error("[generatePasswordRecoveryToken] ", err);
            throw new Error(err);
        }
    }
    async fetchUserData(userName) {
        try {
            return this.UserModel.findOne({ userName })
                .populate('company')
                .populate({
                path: 'role',
                populate: {
                    path: 'permissions',
                    model: 'Permission',
                },
            });
        }
        catch (err) {
            console.error("[fetchUserData] ", err);
            throw new Error(err);
        }
    }
    async notifyWebsockets() {
        try {
            console.log('[WS_USERS] Actualizacion');
            this.websocketService.sendMessageBroadcast(websocketEvents_1.WS_USERS, true);
        }
        catch (err) {
            console.error('[Websocket] Emit error - buffer free');
            console.error("[Websocket] ", err);
            throw new Error(err);
        }
    }
};
UsersService = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => roles_service_1.RolesService))),
    __param(4, (0, common_1.Inject)('MODEL')),
    __metadata("design:paramtypes", [pagination_service_1.PaginationService,
        roles_service_1.RolesService,
        companies_service_1.CompaniesService,
        websocket_gateway_1.WebsocketGateway,
        mongoose_1.Model])
], UsersService);
exports.UsersService = UsersService;
//# sourceMappingURL=users.service.js.map