"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebsocketGateway = void 0;
const common_1 = require("@nestjs/common");
const websockets_1 = require("@nestjs/websockets");
const chalk_1 = require("chalk");
const websocket_config_1 = require("./websocket-config");
const socket_io_1 = require("socket.io");
const https = require("https");
const fs = require("fs");
let WebsocketGateway = class WebsocketGateway {
    constructor(options) {
        this.options = options;
        this.logger = new common_1.Logger(`socket`);
    }
    async afterInit(server) {
        console.log((0, chalk_1.bgYellow)(`Websocket initialized in mode ${this.options.httpsEnabled == true ? 'HTTPS' : 'HTTP'}`));
        if (this.options.httpsEnabled == false || this.options.httpsEnabled == undefined) {
            this.server = await server.listen(this.options.port);
            console.log((0, chalk_1.bgYellow)('Websocket started on port: ' + this.options.port));
            return;
        }
        console.log((0, chalk_1.bgYellow)(`Websocket cert: ${this.options.cert}`));
        console.log((0, chalk_1.bgYellow)(`Websocket key: ${this.options.key}`));
        const options = {
            cert: fs.readFileSync(`${this.options.cert}`),
            key: fs.readFileSync(`${this.options.key}`),
        };
        const socketOptions = {
            path: "",
            cors: {
                origin: '*',
                methods: ["GET", "POST"],
                credentials: true
            },
            allowEIO3: true
        };
        this.server = https.createServer(options);
        this.io = new socket_io_1.Server(this.server, socketOptions);
        this.io.on("connection", (socket) => {
            console.log((0, chalk_1.bgGreen)('Client connected: ', socket['handshake'].headers.origin));
            socket.on("disconnect", (reason) => {
                console.log((0, chalk_1.bgRed)('Client disconnected: ', socket['handshake'].headers.origin));
            });
        });
        this.server.listen(this.options.port);
        console.log((0, chalk_1.bgYellow)('Websocket started on port: ' + this.options.port));
    }
    handleDisconnect(client) {
        console.log((0, chalk_1.bgRed)('Client disconnected: ', client['handshake'].headers.origin));
    }
    async handleConnection(client) {
        console.log((0, chalk_1.bgGreen)('Client connected: ', client['handshake'].headers.origin));
    }
    sendMessageBroadcast(message, data) {
        if (this.options.httpsEnabled == false || this.options.httpsEnabled == undefined) {
            data == null ? this.server.emit(message) : this.server.emit(message, data);
            return;
        }
        data == null ? this.io.emit(message) : this.io.emit(message, data);
    }
};
__decorate([
    (0, websockets_1.WebSocketServer)(),
    __metadata("design:type", Object)
], WebsocketGateway.prototype, "server", void 0);
__decorate([
    (0, websockets_1.WebSocketServer)(),
    __metadata("design:type", Object)
], WebsocketGateway.prototype, "io", void 0);
WebsocketGateway = __decorate([
    (0, common_1.Injectable)(),
    (0, websockets_1.WebSocketGateway)({ cors: true, allowedHeaders: "*", origin: "*", }),
    __param(0, (0, common_1.Inject)('CONFIG_OPTIONS')),
    __metadata("design:paramtypes", [websocket_config_1.WebsocketConfig])
], WebsocketGateway);
exports.WebsocketGateway = WebsocketGateway;
//# sourceMappingURL=websocket.gateway.js.map